<?php

namespace Drupal\Tests\media_entity_slideshow\Functional;

use Drupal\Component\Utility\Color;
use Drupal\Core\Site\Settings;
use Drupal\KernelTests\KernelTestBase;

/**
 * Tests for thumbnail generator.
 *
 * @group media_entity_slideshow
 */
class ThumbnailGeneratorTest extends KernelTestBase {

  /**
   * {@inheritdoc}
   */
  public static $modules = [
    'image',
    'user',
    'field',
    'system',
    'file',
    'media_entity_slideshow',
  ];

  /**
   * Tests thumbnail generation.
   *
   * @dataProvider thumbnailGenerationProvider
   */
  public function testThumbnailGeneration($count) {
    $directory = Settings::get('file_public_path');

    $colors = [
      '#1e81b0',
      '#eeeee4',
      '#e28743',
      '#21130d',
    ];
    $thumbnails = [];
    for ($i = 0; $i < $count; $i++) {

      /** @var \Drupal\Core\Image\ImageInterface $image */

      $image = \Drupal::service('image.factory')->get();

      $image->createNew(1000, 1000);
      $resource = $image->getToolkit()->getResource();

      $file_name = "image-$i.png";
      $file_path = $directory . '/' . $file_name;

      $arr = Color::hexToRgb($colors[$i]);
      $color = imagecolorallocate($resource, $arr['red'], $arr['green'], $arr['blue']);
      imagefill($resource, 0, 0, $color);
      $image->save($file_path);

      $thumbnails[] = $file_path;
    }

    \Drupal::service('media_entity_slideshow.thumbnail_generator')
      ->setWidth(1024)
      ->setHeight(1024)
      ->setPadding(2)
      ->create($thumbnails)
      ->save($directory . '/slideshow.png');

    $image = \Drupal::service('image.factory')->get($directory . '/slideshow.png');

    switch ($count) {
      case 4:
        $this->assertLessThan(10, $this->colorDiff($colors[3], imagecolorat($image->getToolkit()->getResource(), 700, 700)));
      case 3:
        $this->assertLessThan(10, $this->colorDiff($colors[2], imagecolorat($image->getToolkit()->getResource(), 50, 700)));
      case 2:
        $this->assertLessThan(10, $this->colorDiff($colors[1], imagecolorat($image->getToolkit()->getResource(), 800, 50)));
      case 1:
        $this->assertLessThan(10, $this->colorDiff($colors[0], imagecolorat($image->getToolkit()->getResource(), 50, 50)));
    }
  }

  /**
   * Calculate the diff between two colors.
   */
  protected function colorDiff($hex_color, $index_color) {

    $r = ($index_color >> 16) & 0xFF;
    $g = ($index_color >> 8) & 0xFF;
    $b = $index_color & 0xFF;

    $arr = Color::hexToRgb($hex_color);

    return abs($arr['red'] - $r) + abs($arr['green'] - $g) + abs($arr['blue'] - $b);
  }

  /**
   * Data provider for testThumbnailGeneration.
   */
  public function thumbnailGenerationProvider() {
    return [
      [1],
      [2],
      [3],
      [4],
    ];
  }

}
